from django.apps import AppConfig


class ApiConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "r_shepard.api"
    verbose_name = "R Shepard"

    def ready(self):
        # Import the admin module here to avoid a startup import loop
        from django.contrib import admin

        # Set a new label for the application
        admin.site.site_header = "My API Administration"
        admin.site.site_title = "My API"



# Generated by Django 5.0.4 on 2024-04-22 08:59

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("api", "0010_alter_container_port"),
    ]

    operations = [
        migrations.AlterField(
            model_name="container",
            name="tailscale_serve_url",
            field=models.URLField(
                default="",
                help_text="Tailscale Serve URL to access this container's RStudio instance from the tailnet",
                max_length=1024,
                null=True,
            ),
        ),
    ]



from django.contrib import messages
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.template.loader import render_to_string
from django.urls import reverse
from django.views.generic import DetailView, ListView, View
from two_factor.views.mixins import OTPRequiredMixin

from .models import Container, Project
from .podman import (
    PodmanError,
    is_container_running,
    remove_container,
    start_container,
    stop_container,
)


class ProjectListView(OTPRequiredMixin, ListView):
    model = Project
    template_name = "project_list.html"
    context_object_name = "projects"


class ProjectDetailView(OTPRequiredMixin, DetailView):
    model = Project
    template_name = "project_detail.html"

    def get(self, request, *args, **kwargs):
        self.object = self.get_object()
        context = self.get_context_data(object=self.object)
        return self.render_to_response(context)

    def post(self, request, *args, **kwargs):
        pass

    def stop_all_project_containers(self):
        # Logic to stop all podman containers for the project
        pass

    context_object_name = "project"


class CreateContainerView(View):
    def post(self, request, *args, **kwargs):
        # Get project from URL
        project = Project.objects.get(pk=self.kwargs["project_pk"])
        # Get values from form
        password = request.POST.get("password")
        ram = int(request.POST.get("ram"))
        cpus = int(request.POST.get("cpus"))

        # Calculate total resources used by all containers of the project
        total_ram = sum(container.ram for container in project.containers.all())
        total_cpus = sum(container.cpus for container in project.containers.all())

        # Check if the project has enough resources
        if len(project.containers.all()) >= project.max_containers:
            messages.error(request, "Maximum number of containers reached.")
            return HttpResponseRedirect(reverse("project_detail", args=[project.pk]))
        if total_ram + ram > project.max_ram or total_cpus + cpus > project.max_cpus:
            messages.error(request, "Not enough resources for this action.")
            return HttpResponseRedirect(reverse("project_detail", args=[project.pk]))

        # Create container instance with project and password
        container = Container.objects.create(
            project=project, password=password, ram=ram, cpus=cpus
        )

        # Start podman container
        try:
            start_container(project, container, password)
            container.is_running = True
            container.save()
        except PodmanError as e:
            container.delete()
            messages.error(request, str(e))
        return HttpResponseRedirect(reverse("project_detail", args=[project.pk]))


class StartContainerView(OTPRequiredMixin, View):
    def post(self, request, *args, **kwargs):
        project = get_object_or_404(Project, pk=kwargs["project_pk"])
        container = get_object_or_404(Container, pk=kwargs["container_pk"])
        try:
            if not is_container_running(project, container):
                start_container(project, container)
        except PodmanError as e:
            messages.error(request, str(e))
            return HttpResponseRedirect(reverse("project_detail", args=[project.pk]))
        container.is_running = True
        container.save()

        html = render_to_string("container_list.html", {"project": project})
        return HttpResponse(html)


class StopContainerView(OTPRequiredMixin, View):
    def post(self, request, *args, **kwargs):
        project = get_object_or_404(Project, pk=kwargs["project_pk"])
        container = get_object_or_404(Container, pk=kwargs["container_pk"])
        if is_container_running(project, container):
            stop_container(project, container)
        container.is_running = False
        container.save()

        html = render_to_string("container_list.html", {"project": project})
        return HttpResponse(html)


class DeleteContainerView(OTPRequiredMixin, View):
    def post(self, request, *args, **kwargs):
        project = get_object_or_404(Project, pk=kwargs["project_pk"])
        container = get_object_or_404(Container, pk=kwargs["container_pk"])
        if is_container_running(project, container):
            remove_container(project, container)
        container.delete()

        html = render_to_string("container_list.html", {"project": project})
        return HttpResponse(html)



# Generated by Django 5.0.4 on 2024-04-20 17:49

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("api", "0003_project_url"),
    ]

    operations = [
        migrations.AddField(
            model_name="project",
            name="rstudio_version",
            field=models.CharField(
                default="latest",
                help_text="The version of the RStudio Rocker container.",
                max_length=255,
            ),
        ),
    ]



# Generated by Django 5.0.4 on 2024-04-24 10:26

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("api", "0013_remove_project_password_container_password"),
    ]

    operations = [
        migrations.AddField(
            model_name="project",
            name="max_containers",
            field=models.PositiveIntegerField(
                default=4,
                help_text="Maximum number of containers that can be run for the project.",
            ),
        ),
    ]



from django.test import TestCase
from django.urls import reverse

from r_shepard.api.models import Project
from r_shepard.tests.utils import UserMixin


class ProjectListViewAdminTest(UserMixin, TestCase):
    def setUp(self):
        super().setUp()
        self.user = self.create_superuser()
        self.enable_otp()
        self.login_user()
        self.url = reverse("project_list")

    def test_project_list_view(self):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "project_list.html")


class ProjectDetailViewAdminTest(UserMixin, TestCase):
    def setUp(self):
        super().setUp()
        self.user = self.create_superuser()
        self.enable_otp()
        self.login_user()

        self.project = Project.objects.create(
            name="Test Project",
            slug="test-project",
            data_path="/path/to/data",
            workspace_path="/path/to/workspace",
            max_containers=4,
            max_ram=32,
            max_cpus=8,
            auto_commit_enabled=False,
            git_repo_url="https://github.com/user/repo.git",
            commit_interval=15,
        )
        self.url = reverse("project_detail", args=[self.project.id])

    def test_project_detail_view(self):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "project_detail.html")
        self.assertEqual(response.context["project"], self.project)



import subprocess
from datetime import timedelta
from pathlib import Path

from celery import shared_task
from django.utils import timezone

from r_shepard.api.models import Container, Project
from r_shepard.api.podman import PodmanError, is_container_running


@shared_task
def auto_commit():
    # Only do this for projects where auto_commit is enabled
    projects = Project.objects.filter(auto_commit_enabled=True)

    for project in projects:
        print(f"Checking '{project.name}'")
        # Check if it's time to commit
        try:
            now = timezone.now()
            if now - project.last_commit_time >= timedelta(
                minutes=project.commit_interval
            ):
                # Add git origin if .git does not exist
                if not project.git_repo_url:
                    print(f"No Git repository URL for project {project.slug}")
                    continue
                if not Path(f"{project.workspace_path}/.git").exists():
                    result = subprocess.run(
                        ["git", "init"],
                        cwd=project.workspace_path,
                        stdout=subprocess.PIPE,
                        stderr=subprocess.STDOUT,
                    )
                    print(result.stdout.decode())
                    result = subprocess.run(
                        ["git", "remote", "add", "origin", project.git_repo_url],
                        cwd=project.workspace_path,
                        stdout=subprocess.PIPE,
                        stderr=subprocess.STDOUT,
                    )
                    print(result.stdout.decode())
                # Commit and push changes
                result = subprocess.run(
                    ["git", "add", "."],
                    cwd=project.workspace_path,
                    stdout=subprocess.PIPE,
                    stderr=subprocess.STDOUT,
                )
                print(result.stdout.decode())
                result = subprocess.run(
                    ["git", "commit", "-m", "Auto-commit"],
                    cwd=project.workspace_path,
                    stdout=subprocess.PIPE,
                    stderr=subprocess.STDOUT,
                )
                print(result.stdout.decode())
                result = subprocess.run(
                    ["git", "push", project.git_repo_url],
                    cwd=project.workspace_path,
                    stdout=subprocess.PIPE,
                    stderr=subprocess.STDOUT,
                )
                print(result.stdout.decode())
                # Update the last commit time
                project.last_commit_time = now
                project.save()
            else:
                print(f"Not time to commit for project {project.slug}")
        except subprocess.CalledProcessError as e:
            print(f"Encountered git error for project {project.slug}: {e}")
        except Exception as e:
            print(f"Failed to auto-commit for project {project.slug}: {e}")


@shared_task
def check_container_status():
    print("Checking container status...")
    for container in Container.objects.all():
        try:
            project = Project.objects.get(id=container.project_id)
            container.is_running = is_container_running(project, container)
            container.save()
        except PodmanError as e:
            print(f"Failed to check container status: {e}")
        else:
            print(f"Container status updated for {container.name}")



# Generated by Django 5.0.4 on 2024-07-01 08:27

import r_shepard.api.models
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('api', '0019_alter_project_commit_interval_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='container',
            name='port',
            field=models.PositiveIntegerField(default=8787, help_text="The host port on which the container's RStudio instance is accessible locally. Should only be used for testing as this won't be HTTPS enabled.", unique=True, validators=[r_shepard.api.models.validate_port], verbose_name='Host Port'),
        ),
    ]
